@@ -1,4 +1,5 @@
-// (C) Copyright Jonathan Turkanis 2003.
+// (C) Copyright 2008 CodeRage, LLC (turkanis at coderage dot com)
+// (C) Copyright 2003-2007 Jonathan Turkanis
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt.)
 
@@ -18,6 +19,7 @@
 #include <boost/config.hpp> // STATIC_CONSTANT, STDC_NAMESPACE, 
                             // DINKUMWARE_STDLIB, __STL_CONFIG_H.
 #include <algorithm>                      // min.
+#include <cassert>
 #include <cstdio>                         // EOF.
 #include <cstddef>                        // size_t.
 #include <ctime>                          // std::time_t.
@@ -33,7 +35,9 @@
 #include <boost/iostreams/operations.hpp>
 #include <boost/iostreams/device/back_inserter.hpp>
 #include <boost/iostreams/filter/zlib.hpp>
-#include <boost/iostreams/pipeline.hpp>         
+#include <boost/iostreams/pipeline.hpp>     
+#include <boost/iostreams/putback.hpp>
+#include <boost/throw_exception.hpp>
 
 // Must come last.
 #if defined(BOOST_MSVC)
@@ -46,6 +50,8 @@ namespace std { using ::time_t; }
 #endif
 
 namespace boost { namespace iostreams {
+                    
+//------------------Definitions of constants----------------------------------//
 
 namespace gzip {
 
@@ -58,6 +64,7 @@ const int bad_crc           = 2; // Recorded crc doesn't match data.
 const int bad_length        = 3; // Recorded length doesn't match data.
 const int bad_header        = 4; // Malformed header.
 const int bad_footer        = 5; // Malformed footer.
+const int bad_method        = 6; // Unsupported compression method.
 
 namespace magic {
 
@@ -117,6 +124,8 @@ const int os_unknown        = 255;
 
 } // End namespace gzip.
 
+//------------------Definition of gzip_params---------------------------------//
+
 //
 // Class name: gzip_params.
 // Description: Subclass of zlib_params with an additional field
@@ -134,13 +143,15 @@ struct gzip_params : zlib_params {
                  std::string comment    = "",
                  std::time_t mtime      = 0 )
         : zlib_params(level, method, window_bits, mem_level, strategy),
-          file_name(file_name), mtime(mtime)
+          file_name(file_name), comment(comment), mtime(mtime)
         { }
     std::string  file_name;
     std::string  comment;
     std::time_t  mtime;
 };
 
+//------------------Definition of gzip_error----------------------------------//
+
 //
 // Class name: gzip_error.
 // Description: Subclass of std::ios_base::failure thrown to indicate
@@ -162,6 +173,8 @@ private:
     int zlib_error_code_;
 };
 
+//------------------Definition of gzip_compressor-----------------------------//
+
 //
 // Template name: gzip_compressor
 // Description: Model of OutputFilter implementing compression in the
@@ -185,8 +198,7 @@ public:
     template<typename Source>
     std::streamsize read(Source& src, char_type* s, std::streamsize n)
     {
-        using namespace std;
-        streamsize result = 0;
+        std::streamsize result = 0;
 
         // Read header.
         if (!(flags_ & f_header_done))
@@ -196,7 +208,7 @@ public:
         if (!(flags_ & f_body_done)) {
 
             // Read from basic_zlib_filter.
-            streamsize amt = base_type::read(src, s + result, n - result);
+            std::streamsize amt = base_type::read(src, s + result, n - result);
             if (amt != -1) {
                 result += amt;
                 if (amt < n - result) { // Double-check for EOF.
@@ -234,30 +246,27 @@ public:
     template<typename Sink>
     void close(Sink& snk, BOOST_IOS::openmode m)
     {
-        namespace io = boost::iostreams;
+        if (m == BOOST_IOS::out) {
+            try {
 
-        if (m & BOOST_IOS::out) {
+                // Close zlib compressor.
+                base_type::close(snk, BOOST_IOS::out);
 
-            // Close zlib compressor.
-            base_type::close(snk, BOOST_IOS::out);
+                if (flags_ & f_header_done) {
 
-            if (flags_ & f_header_done) {
+                    // Write final fields of gzip file format.
+                    write_long(this->crc(), snk);
+                    write_long(this->total_in(), snk);
+                }
 
-                // Write final fields of gzip file format.
-                write_long(this->crc(), snk);
-                write_long(this->total_in(), snk);
+            } catch (...) {
+                close_impl();
+                throw;
             }
-
+            close_impl();
+        } else {
+            close_impl();
         }
-        #if BOOST_WORKAROUND(__GNUC__, == 2) && defined(__STL_CONFIG_H) || \
-            BOOST_WORKAROUND(BOOST_DINKUMWARE_STDLIB, == 1) \
-            /**/
-            footer_.erase(0, std::string::npos);
-        #else
-            footer_.clear();
-        #endif
-        offset_ = 0;
-        flags_ = 0;
     }
 private:
     static gzip_params normalize_params(gzip_params p);
@@ -273,7 +282,20 @@ private:
         boost::iostreams::put(next, static_cast<char>(0xFF & (n >> 24)));
     }
 
-    enum flag_type {
+    void close_impl()
+    {
+        #if BOOST_WORKAROUND(__GNUC__, == 2) && defined(__STL_CONFIG_H) || \
+            BOOST_WORKAROUND(BOOST_DINKUMWARE_STDLIB, == 1) \
+            /**/
+            footer_.erase(0, std::string::npos);
+        #else
+            footer_.clear();
+        #endif
+        offset_ = 0;
+        flags_ = 0;
+    }
+
+    enum state_type {
         f_header_done = 1,
         f_body_done = f_header_done << 1,
         f_footer_done = f_body_done << 1
@@ -287,6 +309,81 @@ BOOST_IOSTREAMS_PIPABLE(basic_gzip_compressor, 1)
 
 typedef basic_gzip_compressor<> gzip_compressor;
 
+//------------------Definition of helper templates for decompression----------//
+
+namespace detail {
+
+// Processes gzip headers
+class BOOST_IOSTREAMS_DECL gzip_header {
+public:
+    gzip_header() { reset(); }
+
+    // Members for processing header data
+    void process(char c);
+    bool done() const { return state_ == s_done; }
+    void reset();
+
+    // Members for accessing header data
+    std::string file_name() const { return file_name_; }
+    std::string comment() const { return comment_; }
+    bool text() const { return (flags_ & gzip::flags::text) != 0; }
+    int os() const { return os_; }
+    std::time_t mtime() const { return mtime_; }
+private:
+    enum state_type {
+        s_id1       = 1,
+        s_id2       = s_id1 + 1,
+        s_cm        = s_id2 + 1,
+        s_flg       = s_cm + 1,
+        s_mtime     = s_flg + 1,
+        s_xfl       = s_mtime + 1,
+        s_os        = s_xfl + 1,
+        s_xlen      = s_os + 1,
+        s_extra     = s_xlen + 1,
+        s_name      = s_extra + 1,
+        s_comment   = s_name + 1,
+        s_hcrc      = s_comment + 1,
+        s_done      = s_hcrc + 1
+    };
+    std::string  file_name_;
+    std::string  comment_;
+    int          os_;
+    std::time_t  mtime_;
+    int          flags_;
+    int          state_;
+    int          offset_;  // Offset within fixed-length region.
+    int          xlen_;    // Bytes remaining in extra field.
+};
+
+// Processes gzip footers
+class BOOST_IOSTREAMS_DECL gzip_footer {
+public:
+    gzip_footer() { reset(); }
+    
+    // Members for processing footer data
+    void process(char c);
+    bool done() const { return state_ == s_done; }
+    void reset();
+    
+    // Members for accessing footer data
+    zlib::ulong crc() const { return crc_; }
+    zlib::ulong uncompressed_size() const { return isize_; }
+private:
+    enum state_type {
+        s_crc     = 1,
+        s_isize   = s_crc + 1,
+        s_done    = s_isize + 1
+    };
+    zlib::ulong  crc_;
+    zlib::ulong  isize_;
+    int          state_;
+    int          offset_; 
+};
+
+} // End namespace boost::iostreams::detail.
+
+//------------------Definition of basic_gzip_decompressor---------------------//
+
 //
 // Template name: basic_gzip_decompressor
 // Description: Model of InputFilter implementing compression in the
@@ -294,6 +391,9 @@ typedef basic_gzip_compressor<> gzip_compressor;
 //
 template<typename Alloc = std::allocator<char> >
 class basic_gzip_decompressor : basic_zlib_decompressor<Alloc> {
+private:
+    typedef basic_zlib_decompressor<Alloc>   base_type;
+    typedef typename base_type::string_type  string_type;
 public:
     typedef char char_type;
     struct category
@@ -306,35 +406,72 @@ public:
     template<typename Source>
     std::streamsize read(Source& src, char_type* s, std::streamsize n)
     {
-        if ((flags_ & f_header_read) == 0) {
-            non_blocking_adapter<Source> nb(src);
-            read_header(nb);
-            flags_ |= f_header_read;
-        }
-
-        if ((flags_ & f_footer_read) != 0)
-            return -1;
-        
-        try {
-            std::streamsize result = 0;
-            std::streamsize amt;
-            if ((amt = base_type::read(src, s, n)) != -1) {
-                result += amt;
-                if (amt < n) { // Double check for EOF.
-                    amt = base_type::read(src, s + result, n - result);
-                    if (amt != -1)
+        typedef char_traits<char>  traits_type;
+        std::streamsize            result = 0;
+        peekable_source<Source>    peek(src, putback_);
+        while (result < n && state_ != s_done) {
+            if (state_ == s_start) {
+                state_ = s_header;
+                header_.reset();
+                footer_.reset();
+            }
+            if (state_ == s_header) {
+                int c = boost::iostreams::get(peek);
+                if (traits_type::is_eof(c)) {
+                    boost::throw_exception(gzip_error(gzip::bad_header));
+                } else if (traits_type::would_block(c)) {
+                    break;
+                }
+                header_.process(c);
+                if (header_.done())
+                    state_ = s_body;
+            } else if (state_ == s_body) {
+                try {
+                    std::streamsize amt = 
+                        base_type::read(peek, s + result, n - result);
+                    if (amt != -1) {
                         result += amt;
+                        if (amt < n - result)
+                            break;
+                    } else {
+                        peek.putback(this->unconsumed_input());
+                        state_ = s_footer;
+                    }
+                } catch (const zlib_error& e) {
+                    boost::throw_exception(gzip_error(e));
+                }
+            } else { // state_ == s_footer
+                int c = boost::iostreams::get(peek);
+                if (traits_type::is_eof(c)) {
+                    boost::throw_exception(gzip_error(gzip::bad_footer));
+                } else if (traits_type::would_block(c)) {
+                    break;
+                }
+                footer_.process(c);
+                if (footer_.done()) {
+                    if (footer_.crc() != this->crc())
+                        boost::throw_exception(gzip_error(gzip::bad_crc));
+                    int c = boost::iostreams::get(peek);
+                    if (traits_type::is_eof(c)) {
+                        state_ = s_done;
+                    } else {
+                        peek.putback(c);
+                        base_type::close(peek, BOOST_IOS::in);
+                        state_ = s_start;
+                        header_.reset();
+                        footer_.reset();
+                    }
                 }
             }
-            if (amt == -1) {
-                non_blocking_adapter<Source> nb(src);
-                read_footer(nb);
-                flags_ |= f_footer_read;
-            }
-            return result;
-        } catch (const zlib_error& e) {
-            throw gzip_error(e);
         }
+        if (peek.has_unconsumed_input()) {
+            putback_ = peek.unconsumed_input();
+        } else {
+            putback_.clear();
+        }
+        return result != 0 || state_ != s_done ?
+            result :
+            -1;
     }
 
     template<typename Source>
@@ -342,136 +479,96 @@ public:
     {
         try {
             base_type::close(src, BOOST_IOS::in);
-            flags_ = 0;
         } catch (const zlib_error& e) {
-            throw gzip_error(e);
+            state_ = s_start;
+            header_.reset();
+            footer_.reset();
+            boost::throw_exception(gzip_error(e));
         }
+        state_ = s_start;
     }
 
-    std::string file_name() const { return file_name_; }
-    std::string comment() const { return comment_; }
-    bool text() const { return (flags_ & gzip::flags::text) != 0; }
-    int os() const { return os_; }
-    std::time_t mtime() const { return mtime_; }
+    std::string file_name() const { return header_.file_name(); }
+    std::string comment() const { return header_.comment(); }
+    bool text() const { return header_.text(); }
+    int os() const { return header_.os(); }
+    std::time_t mtime() const { return header_.mtime(); }
 private:
-    typedef basic_zlib_decompressor<Alloc>     base_type;
-    typedef BOOST_IOSTREAMS_CHAR_TRAITS(char)  traits_type;
-    static bool is_eof(int c) { return traits_type::eq_int_type(c, EOF); }
     static gzip_params make_params(int window_bits);
 
+    // Source adapter allowing an arbitrary character sequence to be put back.
     template<typename Source>
-    static uint8_t read_uint8(Source& src, int error)
-     {
-        int c;
-        if ((c = boost::iostreams::get(src)) == EOF || c == WOULD_BLOCK)
-            throw gzip_error(error);
-        return static_cast<uint8_t>(traits_type::to_char_type(c));
-    }
-
-    template<typename Source>
-    static uint32_t read_uint32(Source& src, int error)
-    {
-        uint8_t b1 = read_uint8(src, error);
-        uint8_t b2 = read_uint8(src, error);
-        uint8_t b3 = read_uint8(src, error);
-        uint8_t b4 = read_uint8(src, error);
-        return b1 + (b2 << 8) + (b3 << 16) + (b4 << 24);
-    }
-
-    template<typename Source>
-    std::string read_string(Source& src)
-    {
-        std::string result;
-        while (true) {
-            int c;
-            if (is_eof(c = boost::iostreams::get(src)))
-                throw gzip_error(gzip::bad_header);
-            else if (c == 0)
-                return result;
-            else
-                result += static_cast<char>(c);
-        }
-    }
-
-    template<typename Source>
-    void read_header(Source& src) // Source is non-blocking.
-    {
-        // Reset saved values.
-        #if BOOST_WORKAROUND(__GNUC__, == 2) && defined(__STL_CONFIG_H) || \
-            BOOST_WORKAROUND(BOOST_DINKUMWARE_STDLIB, == 1) \
-            /**/
-            file_name_.erase(0, std::string::npos);
-            comment_.erase(0, std::string::npos);
-        #else
-            file_name_.clear();
-            comment_.clear();
-        #endif
-        os_ = gzip::os_unknown;
-        mtime_ = 0;
-
-        int flags;
-
-        // Read header, without checking header crc.
-        if ( boost::iostreams::get(src) != gzip::magic::id1 ||   // ID1.
-             boost::iostreams::get(src) != gzip::magic::id2 ||   // ID2.
-             is_eof(boost::iostreams::get(src)) ||               // CM.
-             is_eof(flags = boost::iostreams::get(src)) )        // FLG.
+    struct peekable_source {
+        typedef char char_type;
+        struct category : source_tag, peekable_tag { };
+        explicit peekable_source(Source& src, const string_type& putback = "") 
+            : src_(src), putback_(putback), offset_(0)
+            { }
+        std::streamsize read(char* s, std::streamsize n)
         {
-            throw gzip_error(gzip::bad_header);
-        }
-        mtime_ = read_uint32(src, gzip::bad_header);        // MTIME.
-        read_uint8(src, gzip::bad_header);                 // XFL.
-        os_ = read_uint8(src, gzip::bad_header);          // OS.
-        if (flags & boost::iostreams::gzip::flags::text)
-            flags_ |= f_text;
+            std::streamsize result = 0;
 
-        // Skip extra field. (From J. Halleaux; see note at top.)
-        if (flags & gzip::flags::extra) {
-            int length = 
-                static_cast<int>(
-                    read_uint8(src, gzip::bad_header) +
-                    (read_uint8(src, gzip::bad_header) << 8)
-                );
-            // length is garbage if EOF but the loop below will quit anyway.
-            do { }
-            while (length-- != 0 && !is_eof(boost::iostreams::get(src)));
+            // Copy characters from putback buffer
+            std::streamsize pbsize = 
+                static_cast<std::streamsize>(putback_.size());
+            if (offset_ < pbsize) {
+                result = (std::min)(n, pbsize - offset_);
+                BOOST_IOSTREAMS_CHAR_TRAITS(char)::copy(
+                    s, putback_.data() + offset_, result);
+                offset_ += result;
+                if (result == n)
+                    return result;
+            }
+
+            // Read characters from src_
+            std::streamsize amt = 
+                boost::iostreams::read(src_, s + result, n - result);
+            return amt != -1 ? 
+                result + amt : 
+                result ? result : -1;
+        }
+        bool putback(char c)
+        {
+            if (offset_) {
+                putback_[--offset_] = c;
+                return true;
+            } else {
+                return boost::iostreams::putback(src_, c);
+            }
+        }
+        void putback(const string_type& s)
+        {
+            putback_.replace(0, offset_, s);
+            offset_ = 0;
         }
 
-        if (flags & gzip::flags::name)          // Read file name.
-            file_name_ = read_string(src);
-        if (flags & gzip::flags::comment)       // Read comment.
-            comment_ = read_string(src);
-        if (flags & gzip::flags::header_crc) {  // Skip header crc.
-            read_uint8(src, gzip::bad_header);
-            read_uint8(src, gzip::bad_header);
+        // Returns true if some characters have been putback but not re-read.
+        bool has_unconsumed_input() const 
+        {
+            return offset_ < static_cast<std::streamsize>(putback_.size());
         }
-    }
 
-    template<typename Source>
-    void read_footer(Source& src)
-    {
-        typename base_type::string_type footer = 
-            this->unconsumed_input();
-        int c;
-        while (!is_eof(c = boost::iostreams::get(src)))
-            footer += c;
-        detail::range_adapter<input, std::string> 
-            rng(footer.begin(), footer.end());
-        if (read_uint32(rng, gzip::bad_footer) != this->crc())
-            throw gzip_error(gzip::bad_crc);
-        if (static_cast<int>(read_uint32(rng, gzip::bad_footer)) != this->total_out())
-            throw gzip_error(gzip::bad_length);
-    }
-    enum flag_type {
-        f_header_read  = 1,
-        f_footer_read  = f_header_read << 1,
-        f_text         = f_footer_read << 1
+        // Returns the sequence of characters that have been put back but not re-read.
+        string_type unconsumed_input() const
+        {
+            return string_type(putback_, offset_, putback_.size() - offset_);
+        }
+        Source&          src_;
+        string_type      putback_;
+        std::streamsize  offset_;
     };
-    std::string  file_name_;
-    std::string  comment_;
-    int          os_;
-    std::time_t  mtime_;
-    int          flags_;
+
+    enum state_type {
+        s_start   = 1,
+        s_header  = s_start + 1,
+        s_body    = s_header + 1,
+        s_footer  = s_body + 1,
+        s_done    = s_footer + 1
+    };
+    detail::gzip_header  header_;
+    detail::gzip_footer  footer_;
+    string_type          putback_;
+    int                  state_;
 };
 BOOST_IOSTREAMS_PIPABLE(basic_gzip_decompressor, 1)
 
@@ -548,10 +645,9 @@ template<typename Alloc>
 std::streamsize basic_gzip_compressor<Alloc>::read_string
     (char* s, std::streamsize n, std::string& str)
 {
-    using namespace std;
-    streamsize avail =
-        static_cast<streamsize>(str.size() - offset_);
-    streamsize amt = (std::min)(avail, n);
+    std::streamsize avail =
+        static_cast<std::streamsize>(str.size() - offset_);
+    std::streamsize amt = (std::min)(avail, n);
     std::copy( str.data() + offset_,
                str.data() + offset_ + amt,
                s );
@@ -570,7 +666,7 @@ template<typename Alloc>
 basic_gzip_decompressor<Alloc>::basic_gzip_decompressor
     (int window_bits, int buffer_size)
     : base_type(make_params(window_bits), buffer_size),
-      os_(gzip::os_unknown), mtime_(0), flags_(0)
+      state_(s_start)
     { }
 
 template<typename Alloc>
