@@ -3,7 +3,7 @@
 
     http://www.boost.org/
 
-    Copyright (c) 2001-2007 Hartmut Kaiser. Distributed under the Boost
+    Copyright (c) 2001-2010 Hartmut Kaiser. Distributed under the Boost
     Software License, Version 1.0. (See accompanying file
     LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 =============================================================================*/
@@ -11,18 +11,18 @@
 #if !defined(CPP_GRAMMAR_HPP_FEAEBC2E_2734_428B_A7CA_85E5A415E23E_INCLUDED)
 #define CPP_GRAMMAR_HPP_FEAEBC2E_2734_428B_A7CA_85E5A415E23E_INCLUDED
 
-#include <boost/spirit/core.hpp>
-#include <boost/spirit/tree/parse_tree.hpp>
-#include <boost/spirit/tree/parse_tree_utils.hpp>
-#include <boost/spirit/utility/confix.hpp>
-#include <boost/spirit/utility/lists.hpp>
+#include <boost/spirit/include/classic_core.hpp>
+#include <boost/spirit/include/classic_parse_tree.hpp>
+#include <boost/spirit/include/classic_parse_tree_utils.hpp>
+#include <boost/spirit/include/classic_confix.hpp>
+#include <boost/spirit/include/classic_lists.hpp>
 
 #include <boost/wave/wave_config.hpp>
 #include <boost/pool/pool_alloc.hpp>
 
 #if BOOST_WAVE_DUMP_PARSE_TREE != 0
 #include <map>
-#include <boost/spirit/tree/tree_to_xml.hpp>
+#include <boost/spirit/include/classic_tree_to_xml.hpp>
 #endif
 
 #include <boost/wave/token_ids.hpp>
@@ -125,12 +125,12 @@ namespace impl {
 //
 ///////////////////////////////////////////////////////////////////////////////
     struct flush_underlying_parser
-    :   public boost::spirit::parser<flush_underlying_parser>
+    :   public boost::spirit::classic::parser<flush_underlying_parser>
     {
         typedef flush_underlying_parser this_t;
 
         template <typename ScannerT>
-        typename boost::spirit::parser_result<this_t, ScannerT>::type
+        typename boost::spirit::classic::parser_result<this_t, ScannerT>::type
         parse(ScannerT const& scan) const
         {
             scan.first.clear_queue();
@@ -153,7 +153,7 @@ namespace impl {
 // Encapsulation of the C++ preprocessor grammar.
 template <typename TokenT, typename ContainerT>
 struct cpp_grammar : 
-    public boost::spirit::grammar<cpp_grammar<TokenT, ContainerT> >
+    public boost::spirit::classic::grammar<cpp_grammar<TokenT, ContainerT> >
 {
     typedef typename TokenT::position_type  position_type;
     typedef cpp_grammar<TokenT, ContainerT> grammar_type;
@@ -166,29 +166,32 @@ struct cpp_grammar :
     {
     // non-parse_tree generating rule type
         typedef typename ScannerT::iteration_policy_t iteration_policy_t;
-        typedef boost::spirit::match_policy match_policy_t;
+        typedef boost::spirit::classic::match_policy match_policy_t;
         typedef typename ScannerT::action_policy_t action_policy_t;
         typedef 
-            boost::spirit::scanner_policies<
+            boost::spirit::classic::scanner_policies<
                 iteration_policy_t, match_policy_t, action_policy_t> 
             policies_t;
         typedef 
-            boost::spirit::scanner<typename ScannerT::iterator_t, policies_t> 
+            boost::spirit::classic::scanner<typename ScannerT::iterator_t, policies_t> 
             non_tree_scanner_t;
         typedef 
-            boost::spirit::rule<non_tree_scanner_t, boost::spirit::dynamic_parser_tag> 
+            boost::spirit::classic::rule<
+                non_tree_scanner_t, boost::spirit::classic::dynamic_parser_tag> 
             no_tree_rule_type;
 
     // 'normal' (parse_tree generating) rule type
         typedef 
-            boost::spirit::rule<ScannerT, boost::spirit::dynamic_parser_tag> 
+            boost::spirit::classic::rule<
+                ScannerT, boost::spirit::classic::dynamic_parser_tag> 
             rule_type;
 
-        rule_type pp_statement;
-        rule_type include_file, system_include_file, macro_include_file;
+        rule_type pp_statement, macro_include_file;
+//         rule_type include_file, system_include_file;
         rule_type plain_define, macro_definition, macro_parameters;
         rule_type undefine;
-        rule_type ppifdef, ppifndef, ppif, ppelse, ppelif, ppendif;
+        rule_type ppifdef, ppifndef, ppif, ppelif;
+//         rule_type ppelse, ppendif;
         rule_type ppline; 
         rule_type pperror;
         rule_type ppwarning;
@@ -205,14 +208,14 @@ struct cpp_grammar :
         definition(cpp_grammar const &self) 
         {
         // import the spirit and cpplexer namespaces here
-            using namespace boost::spirit;
+            using namespace boost::spirit::classic;
             using namespace boost::wave;
             using namespace boost::wave::util;
 
         // set the rule id's for later use
             pp_statement.set_id(BOOST_WAVE_PP_STATEMENT_ID);
-            include_file.set_id(BOOST_WAVE_INCLUDE_FILE_ID);
-            system_include_file.set_id(BOOST_WAVE_SYSINCLUDE_FILE_ID);
+//             include_file.set_id(BOOST_WAVE_INCLUDE_FILE_ID);
+//             system_include_file.set_id(BOOST_WAVE_SYSINCLUDE_FILE_ID);
             macro_include_file.set_id(BOOST_WAVE_MACROINCLUDE_FILE_ID);
             plain_define.set_id(BOOST_WAVE_PLAIN_DEFINE_ID);
             macro_parameters.set_id(BOOST_WAVE_MACRO_PARAMETERS_ID);
@@ -222,8 +225,8 @@ struct cpp_grammar :
             ppifndef.set_id(BOOST_WAVE_IFNDEF_ID);
             ppif.set_id(BOOST_WAVE_IF_ID);
             ppelif.set_id(BOOST_WAVE_ELIF_ID);
-            ppelse.set_id(BOOST_WAVE_ELSE_ID);
-            ppendif.set_id(BOOST_WAVE_ENDIF_ID);
+//             ppelse.set_id(BOOST_WAVE_ELSE_ID);
+//             ppendif.set_id(BOOST_WAVE_ENDIF_ID);
             ppline.set_id(BOOST_WAVE_LINE_ID);
             pperror.set_id(BOOST_WAVE_ERROR_ID);
             ppwarning.set_id(BOOST_WAVE_WARNING_ID);
@@ -252,20 +255,20 @@ struct cpp_grammar :
 
             pp_statement
                 =   (   plain_define
-                    |   include_file
-                    |   system_include_file
+//                     |   include_file
+//                     |   system_include_file
                     |   ppif
                     |   ppelif
                     |   ppifndef
                     |   ppifdef
                     |   undefine
-                    |   ppelse
+//                     |   ppelse
                     |   macro_include_file
                     |   ppline
                     |   pppragma
                     |   pperror
                     |   ppwarning
-                    |   ppendif
+//                     |   ppendif
 #if BOOST_WAVE_SUPPORT_MS_EXTENSIONS != 0
                     |   ppregion
                     |   ppendregion
@@ -285,24 +288,24 @@ struct cpp_grammar :
 #endif // !(defined(BOOST_SPIRIT_DEBUG) &&
                 ;
 
-        // #include ...
-            include_file            // include "..."
-                =   ch_p(T_PP_QHEADER) 
-                    [ store_found_directive_type(self.found_directive) ]
-#if BOOST_WAVE_SUPPORT_INCLUDE_NEXT != 0
-                |   ch_p(T_PP_QHEADER_NEXT)
-                    [ store_found_directive_type(self.found_directive) ]
-#endif 
-                ;
+//         // #include ...
+//             include_file            // include "..."
+//                 =   ch_p(T_PP_QHEADER) 
+//                     [ store_found_directive_type(self.found_directive) ]
+// #if BOOST_WAVE_SUPPORT_INCLUDE_NEXT != 0
+//                 |   ch_p(T_PP_QHEADER_NEXT)
+//                     [ store_found_directive_type(self.found_directive) ]
+// #endif 
+//                 ;
 
-            system_include_file     // include <...>
-                =   ch_p(T_PP_HHEADER) 
-                    [ store_found_directive_type(self.found_directive) ]
-#if BOOST_WAVE_SUPPORT_INCLUDE_NEXT != 0
-                |   ch_p(T_PP_HHEADER_NEXT)
-                    [ store_found_directive_type(self.found_directive) ]
-#endif 
-                ;
+//             system_include_file     // include <...>
+//                 =   ch_p(T_PP_HHEADER) 
+//                     [ store_found_directive_type(self.found_directive) ]
+// #if BOOST_WAVE_SUPPORT_INCLUDE_NEXT != 0
+//                 |   ch_p(T_PP_HHEADER_NEXT)
+//                     [ store_found_directive_type(self.found_directive) ]
+// #endif 
+//                 ;
 
             macro_include_file      // include ...anything else...
                 =   no_node_d
@@ -421,13 +424,13 @@ struct cpp_grammar :
                         )
                 ;
 
-            ppelse
-                =   no_node_d
-                    [
-                        ch_p(T_PP_ELSE)
-                        [ store_found_directive_type(self.found_directive) ]
-                    ]
-                ;
+//             ppelse
+//                 =   no_node_d
+//                     [
+//                         ch_p(T_PP_ELSE)
+//                         [ store_found_directive_type(self.found_directive) ]
+//                     ]
+//                 ;
 
             ppelif
                 =   no_node_d
@@ -441,13 +444,13 @@ struct cpp_grammar :
                         )
                 ;
 
-            ppendif
-                =   no_node_d
-                    [
-                        ch_p(T_PP_ENDIF)
-                        [ store_found_directive_type(self.found_directive) ]
-                    ]
-                ;
+//             ppendif
+//                 =   no_node_d
+//                     [
+//                         ch_p(T_PP_ENDIF)
+//                         [ store_found_directive_type(self.found_directive) ]
+//                     ]
+//                 ;
 
         // #line ...
             ppline 
@@ -571,8 +574,8 @@ struct cpp_grammar :
                 ;
 
             BOOST_SPIRIT_DEBUG_TRACE_RULE(pp_statement, TRACE_CPP_GRAMMAR);
-            BOOST_SPIRIT_DEBUG_TRACE_RULE(include_file, TRACE_CPP_GRAMMAR);
-            BOOST_SPIRIT_DEBUG_TRACE_RULE(system_include_file, TRACE_CPP_GRAMMAR);
+//             BOOST_SPIRIT_DEBUG_TRACE_RULE(include_file, TRACE_CPP_GRAMMAR);
+//             BOOST_SPIRIT_DEBUG_TRACE_RULE(system_include_file, TRACE_CPP_GRAMMAR);
             BOOST_SPIRIT_DEBUG_TRACE_RULE(macro_include_file, TRACE_CPP_GRAMMAR);
             BOOST_SPIRIT_DEBUG_TRACE_RULE(plain_define, TRACE_CPP_GRAMMAR);
             BOOST_SPIRIT_DEBUG_TRACE_RULE(macro_definition, TRACE_CPP_GRAMMAR);
@@ -581,8 +584,8 @@ struct cpp_grammar :
             BOOST_SPIRIT_DEBUG_TRACE_RULE(ppifdef, TRACE_CPP_GRAMMAR);
             BOOST_SPIRIT_DEBUG_TRACE_RULE(ppifndef, TRACE_CPP_GRAMMAR);
             BOOST_SPIRIT_DEBUG_TRACE_RULE(ppif, TRACE_CPP_GRAMMAR);
-            BOOST_SPIRIT_DEBUG_TRACE_RULE(ppelse, TRACE_CPP_GRAMMAR);
-            BOOST_SPIRIT_DEBUG_TRACE_RULE(ppelif, TRACE_CPP_GRAMMAR);
+//             BOOST_SPIRIT_DEBUG_TRACE_RULE(ppelse, TRACE_CPP_GRAMMAR);
+//             BOOST_SPIRIT_DEBUG_TRACE_RULE(ppelif, TRACE_CPP_GRAMMAR);
             BOOST_SPIRIT_DEBUG_TRACE_RULE(ppendif, TRACE_CPP_GRAMMAR);
             BOOST_SPIRIT_DEBUG_TRACE_RULE(ppline, TRACE_CPP_GRAMMAR);
             BOOST_SPIRIT_DEBUG_TRACE_RULE(pperror, TRACE_CPP_GRAMMAR);
@@ -618,9 +621,9 @@ struct cpp_grammar :
 #if BOOST_WAVE_DUMP_PARSE_TREE != 0
 // helper function and data to get readable names of the rules known to us
     struct map_ruleid_to_name :
-        public std::map<boost::spirit::parser_id, std::string> 
+        public std::map<boost::spirit::classic::parser_id, std::string> 
     {
-        typedef std::map<boost::spirit::parser_id, std::string> base_type;
+        typedef std::map<boost::spirit::classic::parser_id, std::string> base_type;
 
         void init_rule_id_to_name_map(cpp_grammar const &self)
         {
@@ -630,8 +633,8 @@ struct cpp_grammar :
             } 
             init_ruleid_name_map[] = {
                 { BOOST_WAVE_PP_STATEMENT_ID, "pp_statement" },
-                { BOOST_WAVE_INCLUDE_FILE_ID, "include_file" },
-                { BOOST_WAVE_SYSINCLUDE_FILE_ID, "system_include_file" },
+//                 { BOOST_WAVE_INCLUDE_FILE_ID, "include_file" },
+//                 { BOOST_WAVE_SYSINCLUDE_FILE_ID, "system_include_file" },
                 { BOOST_WAVE_MACROINCLUDE_FILE_ID, "macro_include_file" },
                 { BOOST_WAVE_PLAIN_DEFINE_ID, "plain_define" },
                 { BOOST_WAVE_MACRO_PARAMETERS_ID, "macro_parameters" },
@@ -641,8 +644,8 @@ struct cpp_grammar :
                 { BOOST_WAVE_IFNDEF_ID, "ppifndef" },
                 { BOOST_WAVE_IF_ID, "ppif" },
                 { BOOST_WAVE_ELIF_ID, "ppelif" },
-                { BOOST_WAVE_ELSE_ID, "ppelse" },
-                { BOOST_WAVE_ENDIF_ID, "ppendif" },
+//                 { BOOST_WAVE_ELSE_ID, "ppelse" },
+//                 { BOOST_WAVE_ENDIF_ID, "ppendif" },
                 { BOOST_WAVE_LINE_ID, "ppline" },
                 { BOOST_WAVE_ERROR_ID, "pperror" },
                 { BOOST_WAVE_WARNING_ID, "ppwarning" },
@@ -660,7 +663,7 @@ struct cpp_grammar :
         // initialize parser_id to rule_name map
             for (int i = 0; 0 != init_ruleid_name_map[i].parser_id; ++i)
                 base_type::insert(base_type::value_type(
-                    boost::spirit::parser_id(init_ruleid_name_map[i].parser_id), 
+                    boost::spirit::classic::parser_id(init_ruleid_name_map[i].parser_id), 
                     std::string(init_ruleid_name_map[i].rule_name))
                 );
         }
@@ -678,11 +681,11 @@ struct cpp_grammar :
 //
 ///////////////////////////////////////////////////////////////////////////////
 template <typename NodeFactoryT, typename IteratorT, typename ParserT>
-inline boost::spirit::tree_parse_info<IteratorT, NodeFactoryT>
+inline boost::spirit::classic::tree_parse_info<IteratorT, NodeFactoryT>
 parsetree_parse(IteratorT const& first_, IteratorT const& last,
-    boost::spirit::parser<ParserT> const& p)
+    boost::spirit::classic::parser<ParserT> const& p)
 {
-    using namespace boost::spirit;
+    using namespace boost::spirit::classic;
     
     typedef pt_match_policy<IteratorT, NodeFactoryT> pt_match_policy_type;
     typedef scanner_policies<iteration_policy, pt_match_policy_type>
@@ -712,7 +715,7 @@ parsetree_parse(IteratorT const& first_, IteratorT const& last,
 
 template <typename LexIteratorT, typename TokenContainerT>
 BOOST_WAVE_GRAMMAR_GEN_INLINE 
-boost::spirit::tree_parse_info<
+boost::spirit::classic::tree_parse_info<
     LexIteratorT, 
     typename cpp_grammar_gen<LexIteratorT, TokenContainerT>::node_factory_type
 >
@@ -721,7 +724,7 @@ cpp_grammar_gen<LexIteratorT, TokenContainerT>::parse_cpp_grammar (
     position_type const &act_pos, bool &found_eof,
     token_type &found_directive, token_container_type &found_eoltokens)
 {
-    using namespace boost::spirit;
+    using namespace boost::spirit::classic;
     using namespace boost::wave;
     
     cpp_grammar<token_type, TokenContainerT> g(found_eof, found_directive, found_eoltokens);
